@@ -5,10 +5,10 @@ import logging
 import sys
 from os import path
 import argparse
+import inspect
 from pySRSLockin.driver import SRS_Lockin
 from pySRSLockin.plots import PlotObject
 
-
 graphics_dir = path.join(path.dirname(__file__), 'graphics')
 
 class interface():
@@ -67,6 +67,8 @@ class interface():
             self.logger.addHandler(ch)
         self.logger.propagate = False
  
+    ### Functions to create GUI
+
     def create_gui(self,plot=False): 
         '''
             plot, boolean
@@ -219,6 +221,7 @@ class interface():
         self.plot_window.setWindowTitle(f"Powermeter: {self.connected_device_name}")
         self.plot_window.show()
         
+### END functions to create GUI
 
 ### GUI Events Functions
 
@@ -300,6 +303,8 @@ class interface():
 
 ### END GUI Events Functions
 
+### Functions to talk with low-level driver
+
     def connect_device(self):
         device_full_name = self.combo_Devices.currentText() # Get the device name from the combobox
         if(device_full_name==''): # Check  that the name is not empty
@@ -452,7 +457,7 @@ class interface():
         self.continuous_read = True #Until this variable is set to True, the function Update will be repeated continuosly 
         self.logger.info(f"Starting reading from device {self.connected_device_name}...")
         # Call the function self.Update(), which will do some suff (read data from device and store it in a global variable) and then call itself continuosly until the variable self.ContinuousRead is set back to 0
-        self.Update()
+        self.update()
         return
  
     def pause_reading(self):
@@ -466,13 +471,13 @@ class interface():
         #Sets self.ContinuousRead to 0 (this will force the function Update() to stop calling itself)
         self.continuous_read = False
         self.stored_powers = []
-        self.Update() #We call one more time the self.Update() function to make sure plots is cleared. Since self.continuous_read is already set to False, Update() will not acquire data anymore
+        self.update() #We call one more time the self.Update() function to make sure plots is cleared. Since self.continuous_read is already set to False, Update() will not acquire data anymore
         self.logger.info(f"Stopped reading from device {self.connected_device_name}. All stored data have been deleted.")
         self.set_stopped_state() # Change some widgets
         # ...
         return
         
-    def Update(self):
+    def update(self):
         '''
         This routine reads continuosly the data from the device and store the values
         If we are continuosly acquiring (i.e. if self.ContinuousRead = 1) then:
@@ -489,6 +494,8 @@ class interface():
             self.output['theta'] = theta
             for k in self.output.keys():	
                 self.stored_data[k].append(self.output[k])
+            if hasattr(self,'trigger'):
+                QtCore.QTimer.singleShot(int(self.trigger[1]), self.trigger[0]) 
 
             #StoredPowers_np = np.array(self.StoredPowers)
             #Fluctuation = 100 * StoredPowers_np[-10:].ptp(axis=0)  / StoredPowers_np[-10:].mean(axis=0)  
@@ -501,10 +508,22 @@ class interface():
                     self.edit_Output[k].setText(f"{self.output[k]:.2e}")
                 except:
                     self.edit_Output[k].setText(f"nan")
-            QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.Update)
+            QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.update)
            
         return
 
+    def set_trigger(self,external_function,delay=0):
+        '''
+        This method allows to use this device as a trigger for other operations. Every time that this interface object acquires data from the device (i.e. every time 
+        the function self.update is executed with self.continuous_read == True), the function external_function is also called. external_function must be a valid function which does not 
+        require any input parameter.
+        The optional parameter delay sets a delay (in ms) between the call to the function update and the call to the function external_function
+        '''
+        if not(inspect.isfunction(external_function)):
+            self.logger.error(f"Input parameter external_function must be a valid function")    
+        self.logger.info(f"Creating a trigger for this device...")
+        self.trigger = [external_function, delay]
+
     def close(self):
         if (self.instrument.connected == True):
             self.disconnect_device()
@@ -531,7 +550,7 @@ def main():
     Interface.verbose = not(args.decrease_verbose)
     app.aboutToQuit.connect(Interface.close) 
     Interface.create_gui(plot=True)
-    
+    Interface.set_trigger(lambda : print('test'),delay=1000)
     window.show()
     app.exec()# Start the event loop.
 
